# December 18 2011

set.seed(1)

load("DaiData.RData")
cat("Done loading data", fill=TRUE)

Hypergeometric <- function(numballs, numwhiteballs, numballsdrawn, numwhiteballsdrawn){
  return(sum(dhyper(numwhiteballsdrawn:numballsdrawn, numwhiteballs, numballs-numwhiteballs,numballsdrawn)))
  # arguments to dhyper are: num white balls drawn, num white balls in urn, num black balls in urn, number of balls drawn from urn
  # a "white ball" is an observed interaction, a "black ball" is an unobserved interaction. a "drawn ball" is a pair of genes in the gene set.
}


# First, try to replicate their results #
numballs <- sum(different.chroms)/2
numwhiteballs <- sum(gene.inter)/2
percents <- ps <- rep(NA, length(tfs.genes))
for(i in 1:length(tfs.genes)){
  whichers <- match(tfs.genes[[i]], genes)
  whichers <- whichers[!is.na(whichers)] # Some of these are NA because some TFs regulated genes that are not "mapped", according to Zhiming Dai
  numballsdrawn <- sum(different.chroms[whichers,whichers])/2
  numwhiteballsdrawn <- sum(gene.inter[whichers,whichers])/2
  percents[i] <- numwhiteballsdrawn/numballsdrawn
  ps[i] <- Hypergeometric(numballs,numwhiteballs,numballsdrawn,numwhiteballsdrawn)
}

theirpvalues <- read.table("TheirPvalues.txt", header=TRUE, sep="\t", stringsAsFactors=FALSE)
theirpvalues[,4] <- as.numeric(theirpvalues[,4])
pdf("PanelC.pdf")
hist(ps, breaks=20, main="(c)", xlab="", ylab="", cex.main=3, col="pink")
dev.off()
cat("Our Pvalues are identical to the ones that Z Dai sent me on 10/16/11!!!!! Hooray!!!!!", cor(ps[order(tfs, decreasing=FALSE)], theirpvalues[,4]), fill=TRUE)
postscript("PanelC.ps", width=5, height=5, horizontal=FALSE)
hist(ps, breaks=20, main="(c)", xlab="", ylab="", cex.main=3, col="grey")
dev.off()

# Now, trying to take a resampling approach #

ps.resample <- rep(NA, length(tfs.genes))
for(i in 1:length(tfs.genes)){
  cat(i,fill=FALSE)
  whichers <- match(tfs.genes[[i]], genes)
  whichers <- whichers[!is.na(whichers)] # Some of these are NA because some TFs regulated genes that are not "mapped", according to Zhiming Dai  
  numinter <- sum(gene.inter[whichers,whichers])/2
  numpossible <- sum(different.chroms[whichers,whichers])/2
  numinter.random <- numpossible.random <- rep(NA, 1000)
  for(j in 1:length(numinter.random)){
    randomgeneset <- NULL
    for(k in unique(chroms)){
      randomgeneset <- c(randomgeneset,sample(which(chroms==k), sum(chroms[whichers]==k, na.rm=TRUE)))
    }
    numinter.random[j] <- sum(gene.inter[randomgeneset,randomgeneset])/2
    numpossible.random[j] <- sum(different.chroms[randomgeneset,randomgeneset])/2
  }
  ps.resample[i] <- (mean(numinter.random/numpossible.random >= numinter/numpossible))
} 
# Done computing p-values from resampling approach #
pdf("PanelF.pdf")
hist(ps.resample, breaks=20, main="(f)", xlab="", ylab="", cex.main=3, col="pink")
dev.off()

postscript("PanelF.ps", width=5, height=5, horizontal=FALSE)
hist(ps.resample, breaks=20, main="(f)", xlab="", ylab="", cex.main=3, col="grey")
dev.off()


cat("Correlation between 2 types of p-values", cor(ps.resample, ps,method="spearman"), fill=TRUE)

cat("Creating a table",fill=TRUE)
tab <- write.table(file="DaiResamplePValues.txt", col.names=c("Transcription Factor", "Hypergeometric P-Value", "Resampling-Based P-Value"),
                   cbind(tfs, round(ps,6), ps.resample)[order(tfs,decreasing=FALSE),], sep="\t",
                   row.names=FALSE,quote=FALSE)




# Now, we'll generate 1000 random gene sets and associated p-values #
ps.random <- ps.resample.random <- rep(NA, 1000)
for(iter in 1:length(ps.random)){
  if(iter/10 == floor(iter/10)) cat(iter,fill=FALSE)
  # first, choose a tf at random and find the associated genes
  tf <- sample(1:length(tfs),1)
  whichers <- match(tfs.genes[[tf]], genes)
  whichers <- whichers[!is.na(whichers)] # Some of these are NA because some TFs regulated genes that are not "mapped", according to Zhiming Dai
  # Now draw that number of genes from the full set of genes
  tfsample <-  sample(1:length(genes), length(whichers))
  # Now tfsample is the randomly sampled tf. It's now time to calculate p-values.
  # first, calculate p-values based on hypergeometric
  numballsdrawn <- sum(different.chroms[tfsample,tfsample])/2
  numwhiteballsdrawn <- sum(gene.inter[tfsample,tfsample])/2
  ps.random[iter] <- Hypergeometric(numballs,numwhiteballs,numballsdrawn,numwhiteballsdrawn)
  # Now, calculate resampling p-values
  numinter <- sum(gene.inter[tfsample,tfsample])/2
  numpossible <- sum(different.chroms[tfsample,tfsample])/2
  numinter.random <- numpossible.random <- rep(NA, 1000)
  for(j in 1:length(numinter.random)){
    randomgeneset <- NULL
    for(k in unique(chroms)){
      randomgeneset <- c(randomgeneset,sample(which(chroms==k), sum(chroms[tfsample]==k, na.rm=TRUE)))
    }
    numinter.random[j] <- sum(gene.inter[randomgeneset,randomgeneset])/2
    numpossible.random[j] <- sum(different.chroms[randomgeneset,randomgeneset])/2
  }
  ps.resample.random[iter] <- (mean(numinter.random/numpossible.random >= numinter/numpossible))  
}
pdf("PanelA.pdf")
hist(ps.random, breaks=20, main="(a)", xlab="", ylab="", cex.main=3, col="pink")
dev.off()
pdf("PanelD.pdf")
hist(ps.resample.random, breaks=20, main="(d)", xlab="", ylab="", cex.main=3, col="pink")
dev.off()

cat("KS.test results:", fill=TRUE)
print(ks.test(ps.resample.random, "punif"))


postscript("PanelA.ps", width=5, height=5, horizontal=FALSE)
hist(ps.random, breaks=20, main="(a)", xlab="", ylab="", cex.main=3, col="grey")
dev.off()
postscript("PanelD.ps", width=5, height=5, horizontal=FALSE)
hist(ps.resample.random, breaks=20, main="(d)", xlab="", ylab="", cex.main=3, col="grey")
dev.off()

