
# Load in the interaction data 
loaddata <- TRUE
dohypergeos <- TRUE
doresamples <- TRUE
if(loaddata){
  supptab13 <- read.table("SuppTable13.txt", header=TRUE, sep="\t")
  print(dim(supptab13))
  chroms <- as.numeric(supptab13[,1])
  pos <- as.numeric(supptab13[,2])
  tab <- read.table("interactions_SU1_SU2_fdr0.01_inter.txt", header=TRUE, sep="\t", stringsAsFactors=FALSE)
  inter <- matrix(0, nrow=length(chroms),ncol=length(chroms))
  for(i in 1:nrow(tab)){
    if(i/5000 == floor(i/5000)) cat(i, fill=FALSE)    
    cur <- as.numeric(tab[i,1:4])
    inter[(cur[1]==chroms & cur[2]==pos),(cur[3]==chroms & cur[4]==pos)] <- 1
  }
  inter <- inter+t(inter)
  difftchroms <- 1*(outer(chroms,chroms,"-")!=0)
  numballs <- sum(difftchroms)/2
  numwhiteballs <- sum(inter)/2  
  save(file="Duan.RData", list=ls())
}
if(!loaddata) load("Duan.RData")
# Done loading in the interaction data


filenames <- c("centromeres", "telomeres_all",   "telomeres_close_to_centromere","telomeres_far_from_centromere",
               "origins_nonCDR_early","origins_CDR_late","origins_unchecked_early","origins_checked_late",
               "WolfeScerAncesBreakpoints", "WolfeScerKwalBreakpoints", "trnas", "trna_cluster_bright", "trna_cluster_dim", "trna_cluster_other")
 


if(dohypergeos){
# Computing hypergeometric p-values #
pvalues <- NULL
numwhiteballsdrawn.all <-  numballsdrawn.all <- NULL
for(filename in filenames){
  cat("Starting", filename, fill=TRUE)
  fullname <- paste(filename, ".txt", sep="")
  file <- read.table(fullname, header=FALSE, stringsAsFactors=FALSE)
  windowsize <- 2500
  if(strsplit(filename, "_")[[1]][1] == "telomeres") windowsize <- 20000 # for telomeres only!!!!!!!!!!! should be 2.5 kb for all others
  # First, we will calculate hypergeometric p-values
  numwhiteballsdrawn <- 0
  numballsdrawn <- 0
  for(i in 1:nrow(file)){
    if(i/5000 == floor(i/5000)) cat(i,numwhiteballsdrawn,numballsdrawn,fill=TRUE)
    cur <- as.numeric(file[i,])
    which1 <- (chroms==cur[1] & pos>=(cur[2]-windowsize) & pos<=(cur[3]+windowsize))
    which2 <- (chroms==cur[4] & pos>=(cur[5]-windowsize) & pos<=(cur[6]+windowsize))
    numwhiteballsdrawn <- numwhiteballsdrawn+sum(inter[which1,which2])
    numballsdrawn <- numballsdrawn+sum(which1)*sum(which2)
  }
  pvalues <- c(pvalues,sum(dhyper(numwhiteballsdrawn:numballsdrawn, numwhiteballs,numballs-numwhiteballs,numballsdrawn)))
  numwhiteballsdrawn.all <- c(numwhiteballsdrawn.all,numwhiteballsdrawn)
  numballsdrawn.all <- c(numballsdrawn.all,numballsdrawn)  
}
}

if(doresamples){
# Now, it is time to compute the resampling p-values for the Duan et al paper #
pvalues.resample <- NULL
for(filename in filenames){
  set.seed(1)
  cat("Starting", filename, fill=TRUE)
  fullname <- paste(filename, ".txt", sep="")
  file <- as.matrix(read.table(fullname, header=FALSE, stringsAsFactors=FALSE))
  windowsize <- 2500
  if(strsplit(filename, "_")[[1]][1] == "telomeres") windowsize <- 20000 # for telomeres only!!!!!!!!!!! should be 2.5 kb for all others
  # First, we will calculate hypergeometric p-values
  numwhiteballsdrawn <- 0
  numballsdrawn <- 0
  for(i in 1:nrow(file)){
    if(i/5000 == floor(i/5000)) cat(i,numwhiteballsdrawn,numballsdrawn,fill=TRUE)
    cur <- as.numeric(file[i,])
    which1 <- (chroms==cur[1] & pos>=(cur[2]-windowsize) & pos<=(cur[3]+windowsize))
    which2 <- (chroms==cur[4] & pos>=(cur[5]-windowsize) & pos<=(cur[6]+windowsize))
    numwhiteballsdrawn <- numwhiteballsdrawn+sum(inter[which1,which2])
    numballsdrawn <- numballsdrawn+sum(which1)*sum(which2)
  }
  ratio <- numwhiteballsdrawn/numballsdrawn # Ultimately, I'll be interested just in this ratio
  allsegments <- unique(rbind(file[,1:3], file[,4:6])) # These are the unique regions associated with this set of potentially colocalized regions
  numballsdrawn.rand.all <- numwhiteballsdrawn.rand.all <- NULL
  niter <- 3000
  if(nrow(file)>2000) niter <- 500
  for(iter in 1:niter){
    if(iter/5 == floor(iter/5)) cat(iter,fill=FALSE)
    # Going to create randfile, a matrix just like file, but contains randomly chosen coordinates that are on the same chromosome, and of same length,
    # as the ones in file
    rand.allsegments <- matrix(NA, nrow=nrow(allsegments), ncol=3)
    rand.allsegments[,1] <- allsegments[,1]
    for(i in 1:nrow(rand.allsegments)){
      rand.allsegments[i,2] <- sample(1:(max(pos[chroms==allsegments[i,1]])-diff(allsegments[i,2:3])),1)
      rand.allsegments[i,3] <- rand.allsegments[i,2]+diff(allsegments[i,2:3])
    }
    randfile <- cbind(rand.allsegments[match(apply(file[,1:3], 1, paste, collapse=" ", sep=" "), apply(allsegments, 1, paste, collapse=" ", sep=" ")), ],
                      rand.allsegments[match(apply(file[,4:6], 1, paste, collapse=" ", sep=" "), apply(allsegments, 1, paste, collapse=" ", sep=" ")), ])
    # Done creating randfile, now I'm ready to count balls drawn and white balls drawn, as usual
    numwhiteballsdrawn.rand <- numballsdrawn.rand <- 0
    for(i in 1:nrow(randfile)){
      cur <- as.numeric(randfile[i,])
      which1 <- (chroms==cur[1] & pos>=(cur[2]-windowsize) & pos<=(cur[3]+windowsize))
      which2 <- (chroms==cur[4] & pos>=(cur[5]-windowsize) & pos<=(cur[6]+windowsize))
      numwhiteballsdrawn.rand <- numwhiteballsdrawn.rand+sum(inter[which1,which2])
      numballsdrawn.rand <- numballsdrawn.rand+sum(which1)*sum(which2)
    }
    numballsdrawn.rand.all <- c(numballsdrawn.rand.all,numballsdrawn.rand)
    numwhiteballsdrawn.rand.all <- c(numwhiteballsdrawn.rand.all,numwhiteballsdrawn.rand)
    ratio.rand <- numwhiteballsdrawn.rand.all/numballsdrawn.rand.all # This is the ratio that I care about
  }
  cat(fill=TRUE)
  pvalues.resample <- c(pvalues.resample,mean(ratio.rand >= ratio)) 
  print(pvalues.resample)
  write(file=paste(filename, "_pvalue.txt", sep=""), rbind(c("NumWhiteBallsDrawn", "HypergeoPs","ResamplePs",
          c(numwhiteballsdrawn.all[filenames==filename]/numballsdrawn.all[filenames==filename], pvalues[filenames==filename], pvalues.resample[filenames==filename]))), ncolumns=3)  
}
}



if(doresamples && dohypergeos) source("MakeBarplot.R")
