# December 18, 2011


# Read in the TF data, which contains info about which genes are regulated by which TFs #
cat("Loading in TF data", fill=TRUE)
tab <- read.table("TFsAndRegulatedGenes.txt", header=TRUE, stringsAsFactors=FALSE)
tfs <- names(tab)
tfs.genes <- list()
for(i in 1:ncol(tab)){
  tfs.genes[[i]] <- tab[!is.na(tab[,i]),i]
}
# Done reading in TF data #

# Read in the chromosomal coordinates data #
cat("Loading in chromosomal coordinates data", fill=TRUE)
tab <- read.table("ChromosomalCoordinates.txt", header=TRUE, stringsAsFactors=FALSE, sep="\t")
tab <- tab[,apply(is.na(tab), 2, mean)<1]
segmentID <- as.matrix(tab[,1:3])
tab <- tab[,-(1:3)]
genes <- tab[!is.na(tab[,1]),1]
genes.segments <- list()
for(i in 1:nrow(tab)){
  if(floor(i/100)==(i/100)) cat(i)
  genes.segments[[i]] <- as.numeric(tab[i, !is.na(tab[i,])][-1])
}
# Done reading in the chromosomal coordinates data #

# Read in interaction data -- which segments interact at FDR < 0.01 #
intermat <- as.matrix(read.table("interactions_HindIII_fdr0.01_inter.txt", header=TRUE, sep="\t")[,1:4])
# Intermat contains (chr for site i, locus for site i), (chr for site j, locus for site j) for interacting sites
# Done reading in interaction data #

# Need to first make a segment interaxn matrix, and use that to create a gene interaxn matrix# 

# First making a segment interaxn matrix #
cat("Starting to create segment interaction matrix", fill=TRUE)
seg.inter <- matrix(0, nrow=nrow(segmentID), ncol=nrow(segmentID))
for(i in 1:nrow(intermat)){
  if(floor(i/1000) == (i/1000)) cat(i,fill=FALSE)
  j <- which(segmentID[,3]==intermat[i,2] & segmentID[,2]==intermat[i,1])
  jj <- which(segmentID[,3]==intermat[i,4]& segmentID[,2]==intermat[i,3])
  seg.inter[j,jj] <- seg.inter[jj,j] <- 1
}
# Done making a segment interaction matrix #

# Now need to make a gene interaction marix #
cat("Starting to create gene interaction matrix", fill=TRUE)
gene.inter <- matrix(0, nrow=length(genes), ncol=length(genes))
for(i in 2:length(genes)){
  if(floor(i/100) == (i/100)) cat(i,fill=FALSE)
  for(j in 1:(i-1)){
    interact <- sum(seg.inter[genes.segments[[i]], genes.segments[[j]]])
    if(interact>0) gene.inter[i,j] <- gene.inter[j,i] <- 1
  }
}
# Done making a gene interaction matrix #


chroms <- read.table("AllGeneChromosomes.txt", header=TRUE, sep="\t", stringsAsFactors=FALSE)
print(sum(genes!=chroms[,1]))
chroms <- chroms[,2]
# A binary matrix for whether or not 2 genes are on different chroms #
different.chroms <- 1*(outer(chroms, chroms, "-")!=0)
# Done making the binary matrix for whether or not 2 genes are on different chroms #

cat("Number of interchrom gene interactions I observed;", sum(gene.inter)/2, "Dai and Dai reported 1163762, but in an email to me on 10/16/11 revised this to 1624900", fill=TRUE)
cat("Number of possible interchrom gene interaxns:", sum(different.chroms)/2, "Dai and Dai reported 15335972, but in an email to me on 10/16/11 revised this to 21164988", fill=TRUE)
save(file="DaiData.RData", list=ls())
